const jwt = require("jsonwebtoken")
const bcrypt = require("bcrypt")
const faker = require("faker")
const User = require("../model/User")
const Cart = require('../model/Cart');
const Address = require('../model/Address');
const PromoCode = require("../model/PromoCode")
const auth = require("../auth")
const { validationResult } = require("express-validator")

// Secret Sauce
require("dotenv").config()

exports.deletePromoCode = async (req, res) => {
	try {
		const promoCodeId = req.params.id

		// Use Mongoose to find and remove the promo code
		const deletedPromoCode = await PromoCode.findOneAndDelete({
			_id: promoCodeId,
		})

		if (!deletedPromoCode) {
			return res.status(404).json({ message: "Promo code not found" })
		}

		res.status(200).json({ message: "Promo code deleted successfully" })
	} catch (error) {
		console.error("Error deleting promo code:", error)
		res.status(500).json({
			error: "An error occurred while deleting promo code",
		})
	}
}

exports.showPromoCodes = async (req, res) => {
	try {
		// Fetch all promo codes from the database
		const promoCodes = await PromoCode.find()

		res.status(200).json({ promoCodes })
	} catch (error) {
		console.error("Error fetching promo codes:", error)
		res.status(500).json({
			error: "An error occurred while fetching promo codes",
		})
	}
}

exports.createPromoCode = async (req, res) => {
	try {
		const { code, discountAmount, expirationDate } = req.body

		// Check if the promo code already exists
		const existingPromoCode = await PromoCode.findOne({ code })

		if (existingPromoCode) {
			return res
				.status(400)
				.json({ message: "Promo code already exists" })
		}

		// Create a new promo code
		const newPromoCode = new PromoCode({
			code,
			discountAmount,
			expirationDate,
			// Add more fields as needed
		})

		// Save the promo code to the database
		await newPromoCode.save()

		res.status(201).json({ message: "Promo code created successfully" })
	} catch (error) {
		console.error("Error creating promo code:", error)
		res.status(500).json({ message: "Internal server error" })
	}
}

exports.applyPromoCode = async (req, res) => {
	try {
	  // Validate request using express-validator
	  const errors = validationResult(req);
	  if (!errors.isEmpty()) {
		return res.status(400).json({ errors: errors.array() });
	  }
  
	  // Retrieve promo code from request body
	  const { promoCode } = req.body;
  
	  // Check if promo code exists in the database
	  const existingPromoCode = await PromoCode.findOne({ code: promoCode });
  
	  if (!existingPromoCode) {
		// If promo code does not exist, return an error response
		return res.status(400).json({ message: "Invalid promo code" });
	  }
  
	  // Log the discount amount
	  console.log("Discount Amount:", existingPromoCode.discountAmount);
  
	  // Additional logic to calculate discount and update total price
	  // Assuming totalPrice is provided in the request or can be fetched from somewhere
	  const totalPrice = 100; // Replace this with actual total price
  
	  // Calculate discounted amount based on the discountAmount
	  const discountAmount = existingPromoCode.discountAmount;
	  
	  // Subtract discounted amount from the current total price
	  const updatedTotalPrice = totalPrice - discountAmount;
  
	  // Log the updated total price
	  console.log("Updated Total Price:", updatedTotalPrice);
  
	  // Return success response
	  return res.status(200).json({
		message: "Promo code applied successfully",
		promoCode: existingPromoCode.code,
		discountAmount,
		updatedTotalPrice,
	  });
	} catch (error) {
	  console.error("Error applying promo code:", error);
	  return res.status(500).json({ message: "Internal server error" });
	}
  };
  
exports.resetPassword = async (req, res) => {
	const { email, newPassword, resetToken } = req.body

	try {
		// Check if the reset token is valid (you may store reset tokens in your database)
		const user = await User.findOne({ email, resetToken })

		if (!user) {
			return res
				.status(400)
				.json({ message: "Invalid reset token or email." })
		}

		// Check if the reset token is still valid (e.g., not expired)

		// Reset the password
		user.password = newPassword
		user.resetToken = null // Invalidate the reset token after use
		await user.save()

		return res.status(200).json({ message: "Password reset successful." })
	} catch (error) {
		console.error("Error resetting password:", error)
		return res.status(500).json({ message: "Internal server error." })
	}
}

exports.userDetails = async (req, res) => {
	try {
		// Fetch user details from the database using the authenticated user ID
		const user = await User.findById(req.user.userId)

		if (!user) {
			return res.status(404).json({ message: "User not found" })
		}

		// Check if the token is expired
		const token = req.header("Authorization").replace("Bearer ", "")
		const decoded = jwt.verify(token, process.env.SECRET_SAUCE)

		if (decoded.exp < Date.now() / 1000) {
			// Token is expired
			return res
				.status(401)
				.json({ message: "Unauthorized. Token has expired." })
		}

		// Send the user details in the response
		res.json({
			_id: user._id,
			email: user.email,
			firstName: user.firstName,
			lastName: user.lastName,
			mobileNo: user.mobileNo,
			isAdmin: user.isAdmin,
			orderedProducts: user.orderedProducts,
			// Add other user details you want to include
		})
	} catch (error) {
		console.error("Error fetching user details:", error)

		if (error.name === "TokenExpiredError") {
			return res
				.status(401)
				.json({ message: "Unauthorized. Token has expired." })
		}

		res.status(500).json({ message: "Internal server error" })
	}
}

exports.checkEmail = async (req, res) => {
	try {
		const { email } = req.body

		const existingUser = await User.findOne({ email })

		res.json({ exists: !!existingUser })
	} catch (error) {
		console.error("Error checking email:", error)
		res.status(500).json({ error: "Internal Server Error" })
	}
}

// Controller function for user registration
exports.registerUser = async (req, res) => {
	try {
		const { email, password, firstName, lastName } = req.body

		// If firstName and lastName are not provided, generate default values using faker
		const autoGeneratedFirstName = firstName || faker.name.firstName()
		const autoGeneratedLastName = lastName || faker.name.lastName()

		// Hash the password before saving it
		const hashedPassword = await bcrypt.hash(password, 10)

		// Set mobileNo to null by default
		const newUser = new User({
			email,
			password: hashedPassword,
			firstName: autoGeneratedFirstName,
			lastName: autoGeneratedLastName,
			mobileNo: "",
		})

		await newUser.save()

		res.status(201).json({
			message:
				"User registered successfully. To update account details, access user/update",
		})
	} catch (error) {
		console.error(error)
		res.status(500).json({ message: "Internal server error" })
	}
}

// Controller function for user authentication
exports.authenticateUser = async (req, res) => {
	try {
	  const { email, password } = req.body;
  
	  const user = await User.findOne({ email });
  
	  if (!user) {
		return res.status(401).json({ message: 'Invalid credentials' });
	  }
  
	  const passwordMatch = await bcrypt.compare(password, user.password);
  
	  if (!passwordMatch) {
		return res.status(401).json({ message: 'Invalid credentials' });
	  }
  
	  // Check if there are existing cart items with anonymousUserId
	  const anonymousUserId = req.headers['x-anonymous-user-id'];
	  const existingCart = await Cart.findOne({ userId: anonymousUserId }).populate('items.product');
  
	  if (existingCart) {
		// Update the userId for existing cart items
		existingCart.userId = user._id; // Assuming you have the user object
		await existingCart.save();
	  }
  
	  // Generate JWT token using the function from auth.js
	  const token = auth.generateToken(user._id, user.email, user.isAdmin);
  
	  // Decode JWT token to get expiration time
	  const decodedToken = jwt.decode(token);
  
	  if (decodedToken) {
		const expiration = new Date(decodedToken.exp * 1000); // Convert seconds to milliseconds
		// Log token expiration
		console.log(`Authenticate success. Token will expire on: ${expiration}`);
	  } else {
		console.error('Error decoding token');
	  }
  
	  // Return user details and token
	  res.status(200).json({
		userId: user._id,
		email: user.email,
		firstName: user.firstName,
		lastName: user.lastName,
		mobileNo: user.mobileNo,
		isAdmin: user.isAdmin,
		token: token,
	  });
	} catch (error) {
	  console.error(error);
	  res.status(500).json({ message: 'Internal server error' });
	}
  };

exports.updateUserData = async (req, res) => {
	try {
		const {
			userId,
			newEmail,
			newFirstName,
			newLastName,
			newPassword,
			newMobileNo,
		} = req.body

		const userIdFromToken = req.user.userId

		if (userIdFromToken !== userId) {
			return res.status(403).json({
				message:
					"Permission denied. You can only update your own data.",
			})
		}

		const user = await User.findById(userId)

		if (!user) {
			return res.status(404).json({ message: "User not found" })
		}

		// Update email if provided
		if (newEmail) {
			user.email = newEmail
		}

		// Update firstName if provided
		if (newFirstName) {
			user.firstName = newFirstName
		}

		// Update lastName if provided
		if (newLastName) {
			user.lastName = newLastName
		}

		// Update password if provided
		if (newPassword) {
			const hashedPassword = await bcrypt.hash(newPassword, 10)
			user.password = hashedPassword
		}

		// Update mobileNo if provided
		if (newMobileNo) {
			user.mobileNo = newMobileNo
		}

		// Save the updated user data
		await user.save()

		// Fetch the updated user details
		const updatedUser = await User.findById(userId)

		// Return the complete updated user details in the response
		res.status(200).json({
			message: "User data updated successfully",
			user: {
				_id: updatedUser._id,
				email: updatedUser.email,
				isAdmin: updatedUser.isAdmin,
				firstName: updatedUser.firstName,
				lastName: updatedUser.lastName,
				mobileNo: updatedUser.mobileNo,
			},
		})
	} catch (error) {
		console.error(error)
		res.status(500).json({ message: "Internal server error" })
	}
}

exports.getUserDetails = async (req, res) => {
	try {
		const { userId } = req.body

		const user = await User.findById(userId)

		if (!user) {
			return res.status(404).json({ message: "User not found" })
		}

		// Create a new object with only the desired properties (excluding password)
		const userWithoutPassword = {
			_id: user._id,
			username: user.username,
			email: user.email,
			firstName: user.firstName,
			lastName: user.lastName,
			orderedProducts: user.orderedProducts,
			// Add other properties you want to include in the response
		}

		// Return the user details in the response
		res.status(200).json({
			user: userWithoutPassword,
		})
	} catch (error) {
		console.error(error)
		res.status(500).json({ message: false })
	}
}

exports.setAdmin = async (req, res) => {
	try {
		const { userId } = req.body // assuming userId is sent in the request body

		const user = await User.findById(userId)

		if (!user) {
			return res.status(404).json({ error: "User not found." })
		}

		user.isAdmin = true
		await user.save()

		res.json({ message: "User is now an admin." })
	} catch (error) {
		console.error(error)
		res.status(500).json({ error: "Internal server error" })
	}
}


exports.getAddresses = async (req, res) => {
	try {
	  const userId = req.body.userId; // Assuming you include userId in the URL parameters
  
	  // Validate the presence of the user ID
	  if (!userId) {
		return res.status(400).json({ error: "User ID is required" });
	  }
  
	  // Fetch addresses for the specified user ID
	  const addresses = await Address.find({ userId });
  
	  res.status(200).json(addresses);
	} catch (error) {
	  console.error("Error fetching addresses:", error);
	  res.status(500).json({ error: "Internal Server Error" });
	}
  };
  

exports.createAddress = async (req, res) => {
	try {
	  const { street, zipCode, city, state, country, userId } = req.body;
  
	  // Validate the presence of required fields
	  if (!street || !zipCode || !city || !state || !country || !userId) {
		return res.status(400).json({ error: "Please fill in all required address fields" });
	  }
  
	  // Create the address
	  const address = new Address({
		street,
		zipCode,
		city,
		state,
		country,
		userId,
	  });
  
	  // Save the address to the database
	  const savedAddress = await address.save();
  
	  res.status(201).json(savedAddress);
	} catch (error) {
	  console.error("Error creating address:", error);
	  res.status(500).json({ error: "Internal Server Error" });
	}
  };
  
  exports.deleteAddress = async (req, res) => {
	try {
	  const { id: addressId } = req.params;
  
	  console.log('Received Address ID:', addressId);
  
	  // Check if the address with the given ID exists
	  const existingAddress = await Address.findById(addressId);
  
	  console.log('Existing Address:', existingAddress);
  
	  if (!existingAddress) {
		return res.status(404).json({ message: 'Address not found' });
	  }
  
	  // Perform the deletion of the address
	  await Address.findByIdAndDelete(addressId);
  
	  res.status(200).json({ message: 'Address deleted successfully' });
	} catch (error) {
	  console.error('Error deleting address:', error);
	  res.status(500).json({ message: 'Internal server error' });
	}
  };
  
  // In userController.js or your relevant controller file

exports.updateAddress = async (req, res) => {
	try {
	  const { id: addressId } = req.params;
	  const { street, zipCode, city, state, country } = req.body;
  
	  // Check if the address with the given ID exists
	  const existingAddress = await Address.findById(addressId);
  
	  if (!existingAddress) {
		return res.status(404).json({ message: 'Address not found' });
	  }
  
	  // Update the address fields
	  existingAddress.street = street;
	  existingAddress.zipCode = zipCode;
	  existingAddress.city = city;
	  existingAddress.state = state;
	  existingAddress.country = country;
  
	  // Save the updated address
	  await existingAddress.save();
  
	  res.status(200).json({ message: 'Address updated successfully' });
	} catch (error) {
	  console.error('Error updating address:', error);
	  res.status(500).json({ message: 'Internal server error' });
	}
  };
  