import React, { useState, useEffect } from "react";
import { Form, Button, Table, Alert, Modal } from "react-bootstrap";
import { toast, ToastContainer } from "react-toastify";
import "react-toastify/dist/ReactToastify.css";

const CreatePromoCode = () => {
  const [promoCodes, setPromoCodes] = useState([]);
  const [promoCode, setPromoCode] = useState("");
  const [discountAmount, setDiscountAmount] = useState(""); // Change variable name here
  const [error, setError] = useState(null);
  const [loading, setLoading] = useState(false);
  const [showDeleteModal, setShowDeleteModal] = useState(false);
  const [promoToDelete, setPromoToDelete] = useState(null);

  useEffect(() => {
    fetchPromoCodes();
  }, []);

  const fetchPromoCodes = async () => {
    try {
      setLoading(true);
      const response = await fetch(
        `${process.env.REACT_APP_API_URL}/user/promo-code`,
        {
          method: "GET",
          headers: {
            Authorization: `Bearer ${localStorage.getItem("token")}`,
          },
        }
      );

      if (response.ok) {
        const data = await response.json();
        setPromoCodes(data.promoCodes);
      } else {
        console.error("Failed to fetch promo codes:", response.status);
        setError("Failed to fetch promo codes. Please try again.");
      }
    } catch (error) {
      console.error("Error fetching promo codes:", error);
      setError("An error occurred while fetching promo codes.");
    } finally {
      setLoading(false);
    }
  };

  const notifySuccess = (message) => {
    toast.success(message, {
      position: "bottom-right",
      autoClose: 3000,
    });
  };

  const notifyError = (message) => {
    toast.error(message, {
      position: "bottom-right",
      autoClose: 3000,
    });
  };

  const handleCreatePromoCode = async () => {
    try {
      const response = await fetch(
        `${process.env.REACT_APP_API_URL}/user/create-promo-code`,
        {
          method: "POST",
          headers: {
            "Content-Type": "application/json",
            Authorization: `Bearer ${localStorage.getItem("token")}`,
          },
          body: JSON.stringify({
            code: promoCode, // Provide the 'code' field
            discountAmount,
          }),
        }
      );

      if (response.ok) {
        setPromoCode("");
        setDiscountAmount(""); // Change variable name here
        setError(null);
        fetchPromoCodes();
        notifySuccess("Promo code created successfully");
      } else {
        console.error("Failed to create promo code:", response.status);
        setError("Failed to create promo code. Please try again.");
        notifyError("Failed to create promo code. Please try again.");
      }
    } catch (error) {
      console.error("Error creating promo code:", error);
      setError("An error occurred while creating promo code.");
      notifyError("An error occurred while creating promo code.");
    }
  };

  const handleCloseDeleteModal = () => {
    setPromoToDelete(null);
    setShowDeleteModal(false);
  };

  const handleDeletePromoCode = async (id) => {
    try {
      setLoading(true);
      const response = await fetch(
        `${process.env.REACT_APP_API_URL}/user/delete-promo-code/${id}`,
        {
          method: "DELETE",
          headers: {
            Authorization: `Bearer ${localStorage.getItem("token")}`,
          },
        }
      );

      if (response.ok) {
        fetchPromoCodes();
        notifySuccess("Promo code deleted successfully");
      } else {
        console.error("Failed to delete promo code:", response.status);
        setError("Failed to delete promo code. Please try again.");
        notifyError("Failed to delete promo code. Please try again.");
      }
    } catch (error) {
      console.error("Error deleting promo code:", error);
      setError("An error occurred while deleting promo code.");
      notifyError("An error occurred while deleting promo code.");
    } finally {
      setLoading(false);
      handleCloseDeleteModal();
    }
  };

  const handleShowDeleteModal = (id) => {
    setPromoToDelete(id);
    setShowDeleteModal(true);
  };

  return (
    <div>
      <h2>Create Promo Code</h2>
      {error && <Alert variant="danger">{error}</Alert>}
      <Form>
        <Form.Group controlId="promoCode">
          <Form.Label>Promo Code</Form.Label>
          <Form.Control
            type="text"
            placeholder="Enter promo code"
            value={promoCode}
            onChange={(e) => setPromoCode(e.target.value)}
          />
        </Form.Group>

        <Form.Group controlId="discountAmount">
          <Form.Label>Discount Amount</Form.Label> {/* Change label here */}
          <Form.Control
            type="text"
            placeholder="Enter discount amount"
            value={discountAmount}
            onChange={(e) => setDiscountAmount(e.target.value)}
          />
        </Form.Group>

        <Button variant="primary" onClick={handleCreatePromoCode}>
          Create Promo Code
        </Button>
      </Form>

      <h2>Promo Codes</h2>
      {loading && <p>Loading...</p>}
      <Table striped bordered hover>
        <thead>
          <tr>
            <th>Promo Code</th>
            <th>Discount Amount</th>
            <th>Action</th>
          </tr>
        </thead>
        <tbody>
          {promoCodes.map((promo) => (
            <tr key={promo._id}>
              <td>{promo.code}</td>{" "}
              {/* Display 'code' instead of 'promoCode' */}
              <td>{promo.discountAmount}</td>
              <td>
                <Button
                  variant="danger"
                  onClick={() => handleShowDeleteModal(promo._id)}
                >
                  Delete
                </Button>
              </td>
            </tr>
          ))}
        </tbody>
      </Table>

      {/* Toast Container */}
      <ToastContainer position="bottom-right" autoClose={3000} />

      <Modal show={showDeleteModal} onHide={handleCloseDeleteModal}>
        <Modal.Header closeButton>
          <Modal.Title>Delete Promo Code</Modal.Title>
        </Modal.Header>
        <Modal.Body>
          Are you sure you want to delete this promo code?
        </Modal.Body>
        <Modal.Footer>
          <Button variant="secondary" onClick={handleCloseDeleteModal}>
            Cancel
          </Button>
          <Button
            variant="danger"
            onClick={() => handleDeletePromoCode(promoToDelete)}
          >
            Delete
          </Button>
        </Modal.Footer>
      </Modal>
    </div>
  );
};

export default CreatePromoCode;
