import React, { useState, useEffect } from "react";
import {
  Container,
  Row,
  Col,
  Table,
  Button,
  Collapse,
  Form,
  Modal,
} from "react-bootstrap";
import AddressForm from "../components/AddressForm";
import EditAddress from "../components/EditAddress";
import { jwtDecode } from "jwt-decode";
import { toast, ToastContainer } from "react-toastify";
import "react-toastify/dist/ReactToastify.css";

const CheckoutPage = () => {
  const [cartItems, setCartItems] = useState([]);
  const [totalPrice, setTotalPrice] = useState(0);
  const [shippingFee, setShippingFee] = useState({
    value: 200,
    details: "Fixed shipping fee of ₱200",
  });
  const [promoCode, setPromoCode] = useState("");
  const [personalInfoOpen, setPersonalInfoOpen] = useState(true);
  const [addressOpen, setAddressOpen] = useState(false);
  const [paymentOpen, setPaymentOpen] = useState(false);
  const [isCustomerLoggedIn, setIsCustomerLoggedIn] = useState(false);
  const [userDetails, setUserDetails] = useState(null);
  const [appliedPromoCode, setAppliedPromoCode] = useState(null);
  const [productDetails, setProductDetails] = useState({});
  const [discountAmount, setDiscountAmount] = useState(0);
  const [addAddressFormOpen, setAddAddressFormOpen] = useState(false);
  const [addresses, setAddresses] = useState([]);
  const [selectedAddressId, setSelectedAddressId] = useState(null);
  const [addressFormError, setAddressFormError] = useState(null);


  const handleAddressSelect = (addressId) => {
    setSelectedAddressId(addressId);
  };

  const handleDeleteAddress = async (addressId) => {
    try {
      const token = localStorage.getItem("token");

      const response = await fetch(
        `${process.env.REACT_APP_API_URL}/user/delete-address/${addressId}`,
        {
          method: "DELETE",
          headers: {
            Authorization: `Bearer ${token}`,
          },
        }
      );

      if (response.ok) {
        console.log("Address deleted successfully:", addressId);
        toast.success("Successfully deleted address", {
          position: "bottom-right",
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });
        // Fetch addresses again after deleting an address
        fetchAddresses();
      } else {
        console.error("Failed to delete address:", response.status);
        toast.error("Failed to delete address", {
          position: "bottom-right",
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });
      }
    } catch (error) {
      console.error("Error deleting address:", error);
      toast.error("Error deleting address", {
        position: "bottom-right",
        autoClose: 3000,
        hideProgressBar: false,
        closeOnClick: true,
        pauseOnHover: true,
        draggable: true,
      });
    }
  };

  const [selectedAddress, setSelectedAddress] = useState({
    street: "",
    zipCode: "",
    city: "",
    state: "",
    country: "Philippines",
  });


  const handleContinueAddress = async (e) => {
    e.preventDefault(); // Prevent the default form submission behavior

    // Add validation logic here to check if the required fields are filled
    if (
      !selectedAddress.street ||
      !selectedAddress.zipCode ||
      !selectedAddress.city ||
      !selectedAddress.state
    ) {
      console.error("Please fill in all required address fields");
      // Display an error toast
      toast.error("Please fill in all required address fields", {
        position: "bottom-right",
        autoClose: 3000,
        hideProgressBar: false,
        closeOnClick: true,
        pauseOnHover: true,
        draggable: true,
      });
      return;
    }

    try {
      const token = localStorage.getItem("token");

      // Decode the token to extract user information
      const decodedToken = jwtDecode(token);
      const userId = decodedToken.userId;

      const response = await fetch(
        `${process.env.REACT_APP_API_URL}/user/create-address`,
        {
          method: "POST",
          headers: {
            "Content-Type": "application/json",
            Authorization: `Bearer ${token}`,
          },
          body: JSON.stringify({ ...selectedAddress, userId }), // Include userId in the request body
        }
      );

      if (response.ok) {
        const data = await response.json();
        console.log("Address created successfully:", data);
        // Optionally, you can update the UI or state to reflect the new address
        // Display a success toast
        toast.success("Address created successfully!", {
          position: "bottom-right",
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });
        // Fetch addresses again after creating a new address
        fetchAddresses();

        // Close the forms after creating a new address
        setAddAddressFormOpen(false);
        setPersonalInfoOpen(false);
      } else {
        console.error("Failed to create address:", response.status);
        // Display an error toast
        toast.error("Failed to create address", {
          position: "bottom-right",
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });
      }
    } catch (error) {
      console.error("Error creating address:", error);
      // Display an error toast
      toast.error("Error creating address", {
        position: "bottom-right",
        autoClose: 3000,
        hideProgressBar: false,
        closeOnClick: true,
        pauseOnHover: true,
        draggable: true,
      });
    }
  };

  const fetchAddresses = async () => {
    try {
      const token = localStorage.getItem("token");

      // Decode the JWT token to get user information
      const decoded = jwtDecode(token);
      const userId = decoded.userId;

      const response = await fetch(
        `${process.env.REACT_APP_API_URL}/user/get-address`,
        {
          method: "POST",
          headers: {
            Authorization: `Bearer ${token}`,
            "Content-Type": "application/json",
          },
          body: JSON.stringify({ userId }),
        }
      );

      if (response.ok) {
        const data = await response.json();
        setAddresses(data);

        // Preselect the first address if available
        if (data.length > 0) {
          handleAddressSelect(data[0].id);
        }

        return data; // Return the fetched addresses
      } else {
        console.error("Failed to fetch addresses:", response.status);
        return []; // Return an empty array if fetching fails
      }
    } catch (error) {
      console.error("Error fetching addresses:", error);
      return []; // Return an empty array if an error occurs
    }
  };

  const fetchCartItems = async () => {
    try {
      const token = localStorage.getItem("token");
      const decoded = jwtDecode(token);

      const response = await fetch(
        `${process.env.REACT_APP_API_URL}/cart/user/${decoded.userId}`,
        {
          method: "GET",
          headers: {
            Authorization: `Bearer ${token}`,
          },
        }
      );

      const data = await response.json();

      if (Array.isArray(data.items)) {
        setCartItems(data.items);
      }
    } catch (error) {
      console.error("Error fetching cart items:", error);
    }
  };

  const calculateTotalPrice = () => {
    const totalPriceWithoutShipping = cartItems.reduce((total, item) => {
      const product = productDetails[item.product];
      if (product) {
        return total + product.price * item.quantity;
      }
      return total;
    }, 0);

    // Calculate the discount based on the percentage
    const calculatedDiscount =
      (discountAmount / 100) * totalPriceWithoutShipping;

    // Calculate the total price after applying the discount
    const totalPriceAfterDiscount =
      totalPriceWithoutShipping - calculatedDiscount;

    // Use shippingFee.value in the calculation
    const updatedTotalPrice = totalPriceAfterDiscount + shippingFee.value;

    // Update total price state
    setTotalPrice(updatedTotalPrice);
  };

  const fetchProductDetails = async () => {
    try {
      const token = localStorage.getItem("token");
      const productIds = cartItems.map((item) => item.product);

      const promises = productIds.map(async (productId) => {
        const response = await fetch(
          `${process.env.REACT_APP_API_URL}/product/${productId}`,
          {
            method: "GET",
            headers: {
              Authorization: `Bearer ${token}`,
            },
          }
        );

        if (!response.ok) {
          console.error(
            `Failed to fetch product details for ID ${productId}: ${response.status}`
          );
          return null;
        }

        const productData = await response.json();
        return { [productId]: productData };
      });

      const productDetailsArray = await Promise.all(promises);
      const productDetailsObject = productDetailsArray.reduce(
        (acc, product) => ({ ...acc, ...product }),
        {}
      );
      setProductDetails(productDetailsObject);
    } catch (error) {
      console.error("Error fetching product details:", error);
    }
  };

  const fetchUserProfile = async () => {
    try {
      if (isCustomerLoggedIn) {
        const token = localStorage.getItem("token");

        const response = await fetch(
          `${process.env.REACT_APP_API_URL}/user/details`,
          {
            method: "GET",
            headers: {
              Authorization: `Bearer ${token}`,
            },
          }
        );

        if (response.ok) {
          const data = await response.json();
          setUserDetails(data);
        } else {
          console.error("Failed to fetch user profile:", response.status);
        }
      }
    } catch (error) {
      console.error("Error fetching user profile:", error);
    }
  };

  const checkCustomerLoginStatus = () => {
    const token = localStorage.getItem("token");

    if (token) {
      try {
        const decoded = jwtDecode(token);

        if (decoded && decoded.userId) {
          setIsCustomerLoggedIn(true);
          fetchCartItems(); // Fetch cart items when the user logs in
          return;
        }
      } catch (error) {
        console.error("Error decoding token:", error);
      }
    }

    setIsCustomerLoggedIn(false);
  };

  const handleApplyPromoCode = async () => {
    try {
      const response = await fetch(
        `${process.env.REACT_APP_API_URL}/user/apply-promo-code`,
        {
          method: "POST",
          headers: {
            "Content-Type": "application/json",
            Authorization: `Bearer ${localStorage.getItem("token")}`,
          },
          body: JSON.stringify({
            promoCode,
            discountAmount,
          }),
        }
      );

      if (response.ok) {
        const data = await response.json();

        // Set discount amount obtained from the response
        setDiscountAmount(data.discountAmount || 0);

        // Set the applied promo code
        setAppliedPromoCode(promoCode);

        // Display success notification at the bottom-right
        toast.success("Promo code applied successfully!", {
          position: "bottom-right",
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });

        checkCustomerLoginStatus();
        // Wait for state to update before calculating the total price
        setTimeout(() => {
          calculateTotalPrice();
        }, 0);
      } else {
        console.error("Failed to apply promo code:", response.status);

        // Display error notification at the bottom-right
        toast.error("Failed to apply promo code", {
          position: "bottom-right",
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });
        // Handle error or provide feedback to the user
      }
    } catch (error) {
      console.error("Error applying promo code:", error);

      // Display error notification at the bottom-right
      toast.error("Error applying promo code", {
        position: "bottom-right",
        autoClose: 3000,
        hideProgressBar: false,
        closeOnClick: true,
        pauseOnHover: true,
        draggable: true,
      });
    }
  };

  useEffect(() => {
    checkCustomerLoginStatus();
  }, []);

  useEffect(() => {
    fetchUserProfile();
  }, [isCustomerLoggedIn]);

  useEffect(() => {
    fetchCartItems();
  }, []);

  useEffect(() => {
    fetchProductDetails();
  }, [cartItems]);

  useEffect(() => {
    calculateTotalPrice();
  }, [cartItems, productDetails, shippingFee]);

  useEffect(() => {
    fetchAddresses();
  }, [isCustomerLoggedIn]);

  return (
    <Container className="my-5 text-start">
      <h2>Checkout</h2>
      <Row>
        {/* Left side - Collapsible Forms */}
        <Col xs={12} md={8}>
          {/* Collapsible Table - Personal Information */}
          <div className="border rounded mb-3 p-3">
            <h3>
              <div className="d-flex justify-content-between align-items-start">
                <span className="fs-6">1. Personal Information</span>
                <Button
                  variant="link"
                  onClick={() => setPersonalInfoOpen(!personalInfoOpen)}
                  aria-controls="personalInfoCollapse"
                  aria-expanded={personalInfoOpen}
                >
                  <i
                    className={`bi bi-chevron-${
                      personalInfoOpen ? "up" : "down"
                    }`}
                  ></i>
                </Button>
              </div>
            </h3>
            <Collapse in={personalInfoOpen}>
              <div id="personalInfoCollapse">
                {isCustomerLoggedIn ? (
                  <div>
                    <p>
                      <strong>Name:</strong> {userDetails?.firstName}{" "}
                      {userDetails?.lastName}
                    </p>
                    <p>
                      <strong>Email:</strong> {userDetails?.email}
                    </p>
                    <p>
                      <strong>Mobile Number:</strong> {userDetails?.mobileNo}
                    </p>
                    <p>If you want to update your information</p>
                    <Button variant="primary" href="/update-profile">
                      Update Information
                    </Button>
                  </div>
                ) : (
                  <div className="mx-4">
                    <p>
                      You are not logged in. Please log in or register to
                      proceed.
                    </p>
                    <Button variant="primary" href="/register">
                      Register
                    </Button>
                    <Button className="mx-2" variant="primary" href="/login">
                      Login
                    </Button>
                  </div>
                )}
              </div>
            </Collapse>
          </div>

          {/* Collapsible Table - Address */}
          <div className="border rounded mb-3 p-3">
            <h3>
              <div className="d-flex justify-content-between align-items-start">
                <span className="fs-6">2. Address</span>
                <Button
                  variant="link"
                  onClick={() => setAddressOpen(!addressOpen)}
                  aria-controls="addressCollapse"
                  aria-expanded={addressOpen}
                >
                  <i
                    className={`bi bi-chevron-${addressOpen ? "up" : "down"}`}
                  ></i>
                </Button>
              </div>
            </h3>
            <Collapse in={addressOpen}>
              <div id="addressCollapse">
                {isCustomerLoggedIn ? (
                  <Row>
                    <span className="mb-2" style={{ fontSize: "16px" }}>
                      The selected address will be used both as your personal
                      address (for invoice) and as your delivery address.
                    </span>
                    <Col className="border rounded mb-3 p-3" md={6}>
                      {/* Left column - Add New Address Form Dropdown */}
                      <div>
                        {addAddressFormOpen && (
                          <AddressForm
                            selectedAddress={selectedAddress}
                            setSelectedAddress={setSelectedAddress}
                            handleContinueAddress={handleContinueAddress}
                            addressFormError={addressFormError}
                          />
                        )}

                        <Button
                          className="my-3"
                          variant="primary"
                          onClick={() =>
                            setAddAddressFormOpen(!addAddressFormOpen)
                          }
                        >
                          Add New Address
                        </Button>
                      </div>
                    </Col>

                    <Col md={6}>
                      {/* Right column - My Addresses from addressForm */}
                      {addresses && addresses.length > 0 ? (
                        <Table striped bordered hover>
                          <thead>
                            <tr>
                              <th>My Addresses</th>
                              <th>Actions</th>
                            </tr>
                          </thead>
                          <tbody>
                            {addresses.map((address, index) => (
                              <tr key={index}>
                                <td>
                                  <div className="d-flex justify-content-between align-items-center">
                                    <Form.Check
                                      type="radio"
                                      id={`addressRadio${index}`}
                                      name="addressRadio"
                                      checked={selectedAddressId === address.id}
                                      onChange={() =>
                                        handleAddressSelect(address.id)
                                      }
                                    />
                                    <p className="mx-3">
                                      {address.street}, {address.city},{" "}
                                      {address.state}, {address.zipCode},{" "}
                                      {address.country}
                                    </p>
                                  </div>
                                </td>
                                <td>
                                  <Button
                                    variant="link"
                                    className="text-danger"
                                    onClick={() =>
                                      handleDeleteAddress(address._id)
                                    }
                                  >
                                    <i className="bi bi-trash"></i>
                                  </Button>
                                </td>
                              </tr>
                            ))}
                          </tbody>
                        </Table>
                      ) : (
                        <p>No existing addresses. Add a new address below.</p>
                      )}
                    </Col>
                  </Row>
                ) : (
                  <p>
                    User not logged in. Log in to view and manage addresses.
                  </p>
                )}
              </div>
            </Collapse>
          </div>

          {/* Collapsible Table - Payment */}
          <div className="border rounded mb-3 p-3">
            <h3>
              <div className="d-flex justify-content-between align-items-start">
                <span className="fs-6">4. Payment</span>
                <Button
                  variant="link"
                  onClick={() => setPaymentOpen(!paymentOpen)}
                  aria-controls="paymentCollapse"
                  aria-expanded={paymentOpen}
                >
                  <i
                    className={`bi bi-chevron-${paymentOpen ? "up" : "down"}`}
                  ></i>
                </Button>
              </div>
            </h3>
            <Collapse in={paymentOpen}>
              <div id="paymentCollapse">
                {/* Placeholder content for payment table */}
              </div>
            </Collapse>
          </div>
        </Col>
        {/* Right side - Order Summary */}
        <Col xs={12} md={4} className="text-start">
          <h3>Order Summary</h3>
          <Table striped bordered hover>
            <tbody>
              <tr>
                <td>Total Items</td>
                <td>
                  {cartItems.reduce((total, item) => total + item.quantity, 0)}
                </td>
              </tr>
              <tr>
                <td>Shipping Fee</td>
                <td>{shippingFee.value}</td>
              </tr>
              <tr>
                <td>Shipping Details</td>
                <td>{shippingFee.details}</td>
              </tr>

              <tr>
                <td>Total</td>
                <td>
                  <strong>₱{totalPrice}</strong>
                </td>
              </tr>
              <tr>
                <td>Discount</td>
                <td>
                  <strong>{discountAmount}%</strong>
                </td>
              </tr>
              <tr>
                <td colSpan="2">
                  <Form.Group controlId="promoCode">
                    <Form.Label>Promo Code</Form.Label>
                    <Form.Control
                      type="text"
                      placeholder="Enter promo code"
                      value={promoCode}
                      onChange={(e) => setPromoCode(e.target.value)}
                    />
                    <Button
                      className="mt-3"
                      variant="primary"
                      onClick={handleApplyPromoCode}
                    >
                      Apply Promo Code
                    </Button>
                  </Form.Group>
                  {appliedPromoCode && (
                    <div>
                      <p className="mt-2">
                        Applied Promo Code: <strong>{appliedPromoCode}</strong>
                      </p>
                    </div>
                  )}
                </td>
              </tr>
            </tbody>
          </Table>
        </Col>
      </Row>
      <ToastContainer position="bottom-right" />
    </Container>
  );
};

export default CheckoutPage;
